package com.example.thumbnail_creator;

import android.Manifest;
import android.annotation.TargetApi;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.os.Build;
import android.util.Log;

import java.io.ByteArrayOutputStream;
import java.io.File;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** FlutterPlugin */
public class ThumbnailCreatorPlugin implements MethodCallHandler {
  static Registrar registrar_global;

  public static void registerWith(Registrar registrar) {
    registrar_global = registrar;
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "thumbnail_creator");
    channel.setMethodCallHandler(new ThumbnailCreatorPlugin());
  }

  @TargetApi(Build.VERSION_CODES.M)
  @Override
  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("getThumbnail")) {
      registrar_global.activity().requestPermissions(new String[]{Manifest.permission.READ_EXTERNAL_STORAGE}, 1);

      int reqWidth = call.argument("reqWidth");
      int reqHeight = call.argument("reqHeight");
      String path = call.argument("path");

      File file = new File(path);

      if(file.exists()) {
        new LongOperation(result, reqWidth, reqHeight, path).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
      } else {
        result.error("File does not exist", "ERROR", "ERROR");
      }
    } else {
      result.notImplemented();
    }
  }

  private static Object decodeSampledBitmap(String path, int reqWidth, int reqHeight) {
    String extension = path.substring(path.lastIndexOf('.') + 1).toLowerCase();

    if (!extension.equals("gif")) {
      final BitmapFactory.Options options = new BitmapFactory.Options();
      options.inPreferredConfig = Bitmap.Config.RGB_565;
      options.inDensity = 40;
      options.inJustDecodeBounds = true;
      BitmapFactory.decodeFile(path, options);
      options.inSampleSize = calculateInSampleSize(options, reqWidth, reqHeight);
      options.inJustDecodeBounds = false;
      Bitmap bmp = BitmapFactory.decodeFile(path, options);
      ByteArrayOutputStream stream = new ByteArrayOutputStream();

      switch(extension) {
        case "jpeg":
          bmp.compress(Bitmap.CompressFormat.JPEG, 50, stream);
          break;
        case "jpg":
          bmp.compress(Bitmap.CompressFormat.JPEG, 50, stream);
          break;
        case "png":
          bmp.compress(Bitmap.CompressFormat.PNG, 50, stream);
          break;
        default:
          bmp.compress(Bitmap.CompressFormat.JPEG, 50, stream);
          break;
      }

      byte[] byteArray = stream.toByteArray();
      bmp.recycle();
      
      return byteArray;
    } else {
      //Log.d("YOOOOOT", path);
      return path;
    }
  }

  private static int calculateInSampleSize(BitmapFactory.Options options, int reqWidth, int reqHeight) {
    // Raw height and width of image
    final int height = options.outHeight;
    final int width = options.outWidth;
    int inSampleSize = 1;

    if (height > reqHeight || width > reqWidth) {

      final int halfHeight = height / 2;
      final int halfWidth = width / 2;

      // Calculate the largest inSampleSize value that is a power of 2 and keeps both
      // height and width larger than the requested height and width.
      while ((halfHeight / inSampleSize) >= reqHeight
              && (halfWidth / inSampleSize) >= reqWidth) {
        inSampleSize *= 2;
      }
    }

    return inSampleSize;
  }

  @TargetApi(Build.VERSION_CODES.CUPCAKE)
  private static class LongOperation extends AsyncTask<String, Void, Object> {

    private Result result;
    private int reqWidth, reqHeight;
    private String path;

    private LongOperation(Result result, int reqWidth, int reqHeight, String path){
      this.result = result;
      this.reqWidth = reqWidth;
      this.reqHeight = reqHeight;
      this.path = path;
    }

    @Override
    protected Object doInBackground(String... params) {
      try {
        Object result = decodeSampledBitmap(path, reqWidth, reqHeight);
        return result;
      } catch (Exception e) {
        return e;
      }
    }

    @Override
    protected void onPostExecute(Object content) {
      if (content instanceof Exception) {
        result.error("File could not be decoded", "ERROR", "ERROR");
      } else {
        result.success(content);
      }
    }

    @Override
    protected void onPreExecute() {}

    @Override
    protected void onProgressUpdate(Void... values) {}
  }
}

